//
//  DCManagedObject_v1.h
//  What ToDo
//
//  Created by Devin Coughlin on 9/17/06.
//  Copyright 2006 Objective Satisfaction. All rights reserved.
//

#import <Cocoa/Cocoa.h>


@interface DCManagedObject_v1 : NSManagedObject {
	NSMutableDictionary *_orderInfo;
	
	NSString *_cachedUUID;
	
	NSDate *_insertionDate;
}




- (NSArray *) orderArrayForKey:(NSString *)key;

- (id) objectInKey:(NSString *) key atIndex:(unsigned) index;

- (void) removeObjectFromKey:(NSString *) key atIndex:(unsigned) index;


/* Note: because this method will remove the object in question if it is
   already in the array, it is possible that this method may insert the object
   at index - 1 (if the current index < index). We may wish to return an unsigned int
   here letting the caller know what at index the object was ultimately dropped.
*/

- (void) insertObject:(id)object inKey:(NSString *) key atIndex:(unsigned)index;

- (int) countOfKey:(NSString *)key;

- (unsigned) indexOfObject:(id)object inKey:(NSString *)key;

//FIXME(dcc) why is this method public?
- (void) syncOrderDataFromArrayForKey:(NSString *)key;


- (void) syncDirtyArraysToData;


- (NSString *)cachedUUID;

- (NSDate *)insertionDate; //only set if we have recently been inserted: not kept in database



// subclasses should call super
- (void) willSave;
- (void)didTurnIntoFault;

- (void) awakeCommon;

- (void) awakeFromFetch;


- (void) awakeFromInsert;

- (void) delete;

- (DCManagedObject_v1 *) inverseObjectForKey:(NSString *)key;


//subclasses are meant to override
- (NSString *) orderDataKeyForToManyKey:(NSString *) key;
// default orderDataKey is keyOrderData where key is the value of key

- (NSString *) orderedAccessorForToManyKey:(NSString *)key;
// default orderedAccesor is orderedKey where Key is the capitalized value of key

@end
