//
//  LionelLexer.h
//  Lionel
//
//  Created by Devin Coughlin on 1/4/06.
//  Copyright 2006 Devin Coughlin. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@interface LionelLexer : NSObject {
	int _tabWidth;
	
	BOOL _acceptsSingleQuotedStrings;
	BOOL _acceptsDoubleQuotedStrings;
	
	BOOL _acceptsCStyleComments;
	BOOL _acceptsCPPStyleComments;
	BOOL _acceptsShellStyleComments;
	
	BOOL _acceptsFloatNumbers;
	
	id _delegate;
}

+ (LionelLexer *) lionelLexer;

/* Properties */

- (int) tabWidth;
- (void) setTabWidth: (int) tabWidth;
/* A tab in the input stream advances the column to the nearest multiple of tabWidth. The default tabWidth is 4. */

- (BOOL) acceptsSingleQuotedStrings;
- (void) setAcceptsSingleQuotedStrings: (BOOL) acceptsSingleQuotedStrings;

- (BOOL) acceptsDoubleQuotedStrings;
- (void) setAcceptsDoubleQuotedStrings: (BOOL) acceptsDoubleQuotedStrings;

- (BOOL) acceptsCStyleComments;
- (void) setAcceptsCStyleComments: (BOOL) acceptsCStyleComments;

- (BOOL) acceptsCPPStyleComments;
- (void) setAcceptsCPPStyleComments: (BOOL) acceptsCPPStyleComments;

- (BOOL) acceptsShellStyleComments;
- (void) setAcceptsShellStyleComments: (BOOL) acceptsShellStyleComments;

- (BOOL) acceptsFloatNumbers;
- (void) setAcceptsFloatNumbers: (BOOL) acceptsFloatNumbers;

- (id) delegate;
- (void) setDelegate: (id) delegate;

/* Methods */

- (void) lexData: (NSData *) data atEOF: (BOOL) atEOF;

- (void) stop;

@end

#pragma mark -

@interface NSObject (LionelLexerDelegate)


#pragma mark -
#pragma mark Control

- (void) lexerDidStart: (LionelLexer *) lexer;
- (void) lexerDidEnd: (LionelLexer *) lexer;


#pragma mark -
#pragma mark Word

- (void) lexer: (LionelLexer *) lexer 
lexedWord: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

#pragma mark -
#pragma mark Numbers

- (void) lexer: (LionelLexer *) lexer 
lexedIntegerNumber: (NSString *) lexedString 
	   atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedFloatNumber: (NSString *) lexedString 
	  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedHexadecimalNumber: (NSString *) lexedString 
	  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

#pragma mark -
#pragma mark Whitespace

- (void) lexer: (LionelLexer *) lexer 
lexedSpaceWhitespace: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedTabWhitespace: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedNewlineWhitespace: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

#pragma mark -
#pragma mark Strings

/* Notes on string lexing:
   As a convenience to users of the API, string delimiters (i.e. ',") are 
   stripped from the reported lexedString and escaped characters are unespaced.
   But the input range gives the position of the entire string 
   _including delimiters_ in the input stream.
*/

- (void) lexer: (LionelLexer *) lexer 
lexedSingleQuotedString: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedUnterminatedSingleQuotedString: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedDoubleQuotedString: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedUnterminatedDoubleQuotedString: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;


#pragma mark -
#pragma mark Comments

- (void) lexer: (LionelLexer *) lexer 
lexedShellStyleComment: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;


- (void) lexer: (LionelLexer *) lexer 
lexedCPPStyleComment: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedCStyleComment: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;

- (void) lexer: (LionelLexer *) lexer 
lexedUnterminatedCStyleComment: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;



#pragma mark -
#pragma mark Single Character Tokens
- (void) lexer: (LionelLexer *) lexer 
lexedSingleCharacterToken: (NSString *) lexedString 
  atInputRange: (NSRange) range
		  line: (int) line
		column: (int) column;


#pragma mark -
#pragma mark Errors

/* Called upon error */
- (void) lexerReachedUnexpectedEOF: (LionelLexer *) lexer;
- (void) lexer: (LionelLexer *) lexer reachedErrorAtOffset: (int) offset line: (int) line column: (int) column;




@end