//
//  LionelToken.h
//  Lionel
//
//  Created by Devin Coughlin on 4/8/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>


@interface LionelToken : NSObject {
	NSString *_type;
	NSString *_value;
	
	NSRange _inputRange;
	
	int _line;
	int _column;
}

- (id) initWithType: (NSString *) type value: (NSString *) value atInputRange: (NSRange) range line: (int) line column: (int) column;

- (NSString *) type;
- (NSString *) value;

- (NSRange) inputRange;

- (int) line;
- (int) column;


/* Type helpers */

- (BOOL) isWord;

- (BOOL) isNumber;
- (BOOL) isIntegerNumber;
- (BOOL) isFloatNumber;
- (BOOL) isHexadecimalNumber;

- (BOOL) isWhitespace;
- (BOOL) isSpaceWhitespace;
- (BOOL) isTabWhitespace;
- (BOOL) isNewlineWhitespace;

- (BOOL) isString;
- (BOOL) isSingleQuotedString;
- (BOOL) isDoubleQuotedString;


- (BOOL) isUnterminatedSingleQuotedString;
- (BOOL) isUnterminatedDoubleQuotedString;
/* Note: if the token is an unterminated string, isString will return NO */

- (BOOL) isComment;
- (BOOL) isShellStyleComment;
- (BOOL) isCStyleComment;
- (BOOL) isCPPStyleComment;

- (BOOL) isUnterminatedCStyleComment;
/* Note: again, if the token is an unterminated c style comment, isComment will return NO */

- (BOOL) isSingleCharacterToken;
- (BOOL) isSingleCharacterToken: (NSString *) tokenValue;

- (BOOL) isErrorToken;
@end

/* Token Types */

extern NSString *LionelWordToken;

extern NSString *LionelIntegerNumberToken;
extern NSString *LionelFloatNumberToken;
extern NSString *LionelHexadecimalNumberToken;

extern NSString *LionelSpaceWhitespaceToken;
extern NSString *LionelTabWhitespaceToken;
extern NSString *LionelNewlineWhitespaceToken;

extern NSString *LionelSingleQuotedStringToken;
extern NSString *LionelUnterminatedSingleQuotedStringToken;

extern NSString *LionelDoubleQuotedStringToken;
extern NSString *LionelUnterminatedDoubleQuotedStringToken;

extern NSString *LionelShellStyleCommentToken;
extern NSString *LionelCPPStyleCommentToken;

extern NSString *LionelCStyleCommentToken;
extern NSString *LionelUnterminatedCStyleCommentToken;

extern NSString *LionelSingleCharacterToken;

extern NSString *LionelUnexpectedEOFToken;
extern NSString *LionelErrorToken;